/*
name: qvwm-volume.cc
info: Simple audio volume setting application for qvwm, use it together 
      with universal qvwm indicator. I also suggest to set NO_BORDER_EDGE 
      and maybe NO_TITLE in [Applications] part of ~/.qvwmrc file, also
      maybe NO_TBUTTON, maybe STICKY,...
author: Dusan Halicky (dvh.tosomja@post.sk)
licence: GNU GPL (or any other on your demand)
history: 2005-09-22 - begin of development, first version, based on the STK toolkit
	 2005-10-07 - version 2.0 based on STK v2.0.alpha1
	 2005-11-14 - updated for STK v2.0.alpha2 support
todo: -
bugs: -
*/

// STK size optimization, disable unused components
#define STK_DISABLE_BUTTON
#define STK_DISABLE_IMAGE
#define STK_DISABLE_RADIOBUTTON
#define STK_DISABLE_SHORTCUT

// using STK
#include "../../stk2/stksrc/stk.h"		// use STK (in the directory stksrc)

// audio stuff
#include "myaudio.h"

// components
StkWindow *window1;		// main and only window
StkSlider *slider_master;	// vertical slider component for setting volume
StkSlider *slider_pcm;		// pcm volume
StkLabel *label_master;
StkLabel *label_pcm;
StkShortcut *escape;		// close when press ESC

// volumes (now for master and pcm)
int volume_master = 0;
int volume_pcm = 0;
unsigned char ucvalue=0;	// aux variable, must be global else ioctl cause segmentation fault

void
slider_master_change(void)
{
  // get the slider value in percent (note, zero is up in StkSlider so we use Reverse order function GetValueR)
  int value = slider_master->GetValueR();
  // set the real volume
  myaudio_mixer_set_volume(SOUND_MIXER_VOLUME,value,value);
  // get it again (real settings)
  ucvalue=value;
  myaudio_mixer_get_volume(SOUND_MIXER_VOLUME,&ucvalue,&ucvalue);
  volume_master=value;
}

void
slider_pcm_change(void)
{
  // get the slider value in percent (note, zero is up in StkSlider so we use Reverse order function GetValueR)
  int value = slider_pcm->GetValueR();
  // set the real volume
  myaudio_mixer_set_volume(SOUND_MIXER_PCM,value,value);
  // get it again
  ucvalue=value;
  myaudio_mixer_get_volume(SOUND_MIXER_PCM,&ucvalue,&ucvalue);
  volume_pcm=value;
}

void
escapeCallback(void)
{
  // exit
  window1->Close();
}

int
main ()
{
  // init mixer
  myaudio_mixer_init();
  // get the volume
  unsigned char ia=0,ib=0;
  myaudio_mixer_get_volume(SOUND_MIXER_VOLUME,&ia,&ib);
  volume_master = (ia+ib)/2;
  myaudio_mixer_get_volume(SOUND_MIXER_PCM,&ia,&ib);
  volume_pcm = (ia+ib)/2;
  // init and run simple graphical interface using STK toolkit
  stk_init ();
  // debug - get the mouse pointer in real coordinates
  int moux,mouy,cx,cy;
  Window r,w;
  unsigned int uimask;
  XQueryPointer(stkdisplay,stkdisplay_root,&r,&w,&moux,&mouy,&cx,&cy,&uimask);
  printf("r=0x%X w=0x%X moux=%d mouy=%d cx=%d cy=%d uimask=%d\n",(int)r,(int)w,moux,mouy,cx,cy,uimask);
  // calculate position of the main window, not to be outside the screen
  int wx,wy;
  wx=moux;
  wy=mouy;
  if (wx+110>stkdisplay_width) wx=stkdisplay_width-110-5;
  if (wy+110>stkdisplay_height) wy=stkdisplay_height-110-32;
  printf("wx=%d wy=%d\n",wx,wy);
  // create the window (at the cursor position)
  window1 = new StkWindow (stkdisplay_root, wx, wy, 110, 110, "Volume");
  window1->SetResizeable(false);
//  window1 = new StkWindow (-9, -9, 110, 110, "Volume");
  // master volume slider
  slider_master = new StkSlider (window1, 17, 10, 80, STK_SLIDER_VERTICAL, 0, 100, 1);	// vertical slider, range 0-100, 0 is always at the top of slider
  slider_master->CreateCallback (new StkGlobalCallback(&slider_master_change));  
  slider_master->SetFocus (true);
  label_master = new StkLabel (window1, 10, 95, -1, -1, "Ma&ster", slider_master);
  // pcm volume slider
  slider_pcm = new StkSlider (window1, 67, 10, 80, STK_SLIDER_VERTICAL, 0, 100, 1);
  slider_pcm->CreateCallback (new StkGlobalCallback(&slider_pcm_change));  
  label_pcm = new StkLabel (window1, 70, 95, -1, -1, "&Pcm", slider_pcm);
  // escape shortcut
  escape = new StkShortcut (window1, XK_Escape, 0);
  escape->CreateCallback (new StkGlobalCallback(&escapeCallback));
  // show the window
//  window1->Show ();
  slider_master->SetValueR(volume_master);
  slider_pcm->SetValueR(volume_pcm);
  do{
    stk_next_event();
    window1->components->Update();
    } while (! window1->Closed() );
//  stk_run (window1);
  stk_done ();
  return 0;
}
